@@ -28,6 +28,7 @@
 #include <libaudcore/hook.h>
 #include <libaudcore/i18n.h>
 #include <libaudcore/interface.h>
+#include <libaudcore/mainloop.h>
 #include <libaudcore/playlist.h>
 #include <libaudcore/runtime.h>
 #include <libaudcore/tuple.h>
@@ -49,6 +50,7 @@ static struct {
     int qt;
 } options;
 
+static bool initted = false;
 static Index<PlaylistAddItem> filenames;
 
 static const struct {
@@ -295,7 +297,10 @@ static void do_commands ()
         else if (aud_drct_get_paused ())
             aud_drct_pause ();
     }
+}
 
+static void do_commands_at_idle (void *)
+{
     if (options.show_jump_box && ! options.headless)
         aud_ui_show_jump_to_song ();
     if (options.mainwin && ! options.headless)
@@ -304,6 +309,17 @@ static void do_commands ()
 
 static void main_cleanup ()
 {
+    if (initted)
+    {
+        /* Somebody was naughty and called exit() instead of aud_quit().
+         * aud_cleanup() has not been called yet, so there's no point in running
+         * leak checks.  Note that it's not safe to call aud_cleanup() from the
+         * exit handler, since we don't know what context we're in (we could be
+         * deep inside the call tree of some plugin, for example). */
+        AUDWARN ("exit() called unexpectedly; skipping normal cleanup.\n");
+        return;
+    }
+
     filenames.clear ();
     aud_cleanup_paths ();
     aud_leak_check ();
@@ -360,29 +376,33 @@ int main (int argc, char * * argv)
     signals_init_two ();
 #endif
 
+    initted = true;
     aud_init ();
 
     do_commands ();
 
-    if (check_should_quit ())
-        goto QUIT;
+    if (! check_should_quit ())
+    {
+        QueuedFunc at_idle_func;
+        at_idle_func.queue (do_commands_at_idle, nullptr);
 
-    hook_associate ("playback stop", (HookFunction) maybe_quit, nullptr);
-    hook_associate ("playlist add complete", (HookFunction) maybe_quit, nullptr);
-    hook_associate ("quit", (HookFunction) aud_quit, nullptr);
+        hook_associate ("playback stop", (HookFunction) maybe_quit, nullptr);
+        hook_associate ("playlist add complete", (HookFunction) maybe_quit, nullptr);
+        hook_associate ("quit", (HookFunction) aud_quit, nullptr);
 
-    aud_run ();
+        aud_run ();
 
-    hook_dissociate ("playback stop", (HookFunction) maybe_quit);
-    hook_dissociate ("playlist add complete", (HookFunction) maybe_quit);
-    hook_dissociate ("quit", (HookFunction) aud_quit);
+        hook_dissociate ("playback stop", (HookFunction) maybe_quit);
+        hook_dissociate ("playlist add complete", (HookFunction) maybe_quit);
+        hook_dissociate ("quit", (HookFunction) aud_quit);
+    }
 
-QUIT:
 #ifdef USE_DBUS
     dbus_server_cleanup ();
 #endif
 
     aud_cleanup ();
+    initted = false;
 
     return EXIT_SUCCESS;
 }
