@@ -76,6 +76,12 @@ enum GetMode {
     WaitGuess  // wait for the entry to be scanned; return a best guess on failure
 };
 
+/* Format descriptor returned by playlist_save_formats() */
+struct SaveFormat {
+    String name;         // human-readable format name
+    Index<String> exts;  // supported filename extensions
+};
+
 } // namespace Playlist
 
 typedef bool (* PlaylistFilterFunc) (const char * filename, void * user);
@@ -133,6 +139,10 @@ void aud_playlist_set_active (int playlist);
 /* Returns the number of the active playlist. */
 int aud_playlist_get_active ();
 
+/* Convenience function which adds a new playlist after the active one and then
+ * sets the new one as active.  Returns the number of the new playlist. */
+int aud_playlist_new ();
+
 /* Starts playback of a playlist, resuming from the position last played if
  * possible.  If <playlist> is -1 or if the requested playlist is empty, stops
  * playback.  If <paused> is true, starts playback in a paused state. */
@@ -378,4 +388,9 @@ bool aud_filename_is_playlist (const char * filename);
  * is determined from the file extension.  Returns true on success. */
 bool aud_playlist_save (int playlist, const char * filename, Playlist::GetMode mode);
 
+/* Generates a list of the currently supported formats for saving playlists.
+ * The list should not be cached since it may change as plugins are enabled or
+ * disabled. */
+Index<Playlist::SaveFormat> aud_playlist_save_formats ();
+
 #endif
