@@ -35,7 +35,7 @@
 
 /* Increment this when the format of the plugin-registry file changes.
  * Add 10 if the format changes in a way that will break parse_plugins_fallback(). */
-#define FORMAT 9
+#define FORMAT 10
 
 /* Oldest file format supported by parse_plugins_fallback() */
 #define MIN_FORMAT 2  // "enabled" flag was added in Audacious 2.4
@@ -50,12 +50,13 @@ class PluginHandle
 public:
     String basename, path;
     bool loaded;
-    int timestamp;
+    int timestamp, version;
     PluginType type;
     Plugin * header;
     String name, domain;
     int priority;
-    int has_about, has_configure, enabled;
+    int has_about, has_configure;
+    PluginEnabled enabled;
     Index<PluginWatch> watches;
 
     /* for transport plugins */
@@ -63,24 +64,28 @@ public:
 
     /* for playlist plugins */
     Index<String> exts;
+    int can_save;
 
     /* for input plugins */
     aud::array<InputKey, Index<String>> keys;
     int has_subtunes, writes_tag;
 
     PluginHandle (const char * basename, const char * path, bool loaded,
-     int timestamp, PluginType type, Plugin * header) :
+     int timestamp, int version, PluginType type, Plugin * header) :
         basename (basename),
         path (path),
         loaded (loaded),
         timestamp (timestamp),
+        version (version),
         type (type),
         header (header),
         priority (0),
         has_about (false),
         has_configure (false),
-        enabled (type == PluginType::Transport ||
-         type == PluginType::Playlist || type == PluginType::Input),
+        enabled ((type == PluginType::Transport ||
+         type == PluginType::Playlist || type == PluginType::Input) ?
+         PluginEnabled::Primary : PluginEnabled::Disabled),
+        can_save (false),
         has_subtunes (false),
         writes_tag (false) {}
 
@@ -109,6 +114,7 @@ static constexpr aud::array<InputKey, const char *> input_key_names = {
 };
 
 static aud::array<PluginType, Index<PluginHandle *>> plugins;
+static aud::array<PluginType, Index<PluginHandle *>> compatible;
 static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static StringBuf get_basename (const char * path)
@@ -140,6 +146,8 @@ static void playlist_plugin_save (PluginHandle * plugin, FILE * handle)
 {
     for (const String & ext : plugin->exts)
         fprintf (handle, "ext %s\n", (const char *) ext);
+
+    fprintf (handle, "saves %d\n", plugin->can_save);
 }
 
 static void input_plugin_save (PluginHandle * plugin, FILE * handle)
@@ -158,6 +166,7 @@ static void plugin_save (PluginHandle * plugin, FILE * handle)
 {
     fprintf (handle, "%s %s\n", plugin_type_names[plugin->type], (const char *) plugin->path);
     fprintf (handle, "stamp %d\n", plugin->timestamp);
+    fprintf (handle, "version %d\n", plugin->version);
     fprintf (handle, "name %s\n", (const char *) plugin->name);
 
     if (plugin->domain)
@@ -166,7 +175,7 @@ static void plugin_save (PluginHandle * plugin, FILE * handle)
     fprintf (handle, "priority %d\n", plugin->priority);
     fprintf (handle, "about %d\n", plugin->has_about);
     fprintf (handle, "config %d\n", plugin->has_configure);
-    fprintf (handle, "enabled %d\n", plugin->enabled);
+    fprintf (handle, "enabled %d\n", (int) plugin->enabled);
 
     if (plugin->type == PluginType::Transport)
         transport_plugin_save (plugin, handle);
@@ -195,6 +204,9 @@ void plugin_registry_save ()
         list.clear ();
     }
 
+    for (auto & list : compatible)
+        list.clear ();
+
     fclose (handle);
 }
 
@@ -254,6 +266,9 @@ static void playlist_plugin_parse (PluginHandle * plugin, FILE * handle)
         plugin->exts.append (std::move (value));
         parse_next (handle);
     }
+
+    if (parse_integer ("saves", & plugin->can_save))
+        parse_next (handle);
 }
 
 static void input_plugin_parse (PluginHandle * plugin, FILE * handle)
@@ -302,11 +317,15 @@ static bool plugin_parse (FILE * handle)
     if (! parse_integer ("stamp", & timestamp))
         return false;
 
-    auto plugin = new PluginHandle (basename, String (), false, timestamp, type, nullptr);
-    plugins[type].append (plugin);
-
     parse_next (handle);
 
+    int version = 0;
+    if (parse_integer ("version", & version))
+        parse_next (handle);
+
+    auto plugin = new PluginHandle (basename, String (), false, timestamp, version, type, nullptr);
+    plugins[type].append (plugin);
+
     plugin->name = parse_string ("name");
     if (plugin->name)
         parse_next (handle);
@@ -321,8 +340,13 @@ static bool plugin_parse (FILE * handle)
         parse_next (handle);
     if (parse_integer ("config", & plugin->has_configure))
         parse_next (handle);
-    if (parse_integer ("enabled", & plugin->enabled))
+
+    int enabled;
+    if (parse_integer ("enabled", & enabled))
+    {
+        plugin->enabled = (PluginEnabled) enabled;
         parse_next (handle);
+    }
 
     if (type == PluginType::Transport)
         transport_plugin_parse (plugin, handle);
@@ -369,9 +393,9 @@ static void parse_plugins_fallback (FILE * handle)
             return;
 
         // setting timestamp to zero forces a rescan
-        auto plugin = new PluginHandle (basename, String (), false, 0, type, nullptr);
+        auto plugin = new PluginHandle (basename, String (), false, 0, 0, type, nullptr);
         plugins[type].append (plugin);
-        plugin->enabled = enabled;
+        plugin->enabled = (PluginEnabled) enabled;
     }
 }
 
@@ -431,18 +455,29 @@ void plugin_registry_prune ()
         return true;
     };
 
-    for (auto & list : plugins)
+    auto check_incompatible = [] (PluginHandle * plugin)
     {
-        list.remove_if (check_not_found);
-        list.sort (plugin_compare, nullptr);
+        if (plugin_check_flags (plugin->version))
+            return false;
+
+        AUDINFO ("Incompatible plugin flags: %s\n", (const char *) plugin->basename);
+        return true;
+    };
+
+    for (auto type : aud::range<PluginType> ())
+    {
+        plugins[type].remove_if (check_not_found);
+        plugins[type].sort (plugin_compare, nullptr);
+        compatible[type].insert (plugins[type].begin (), 0, plugins[type].len ());
+        compatible[type].remove_if (check_incompatible);
     }
 }
 
 /* Note: If there are multiple plugins with the same basename, this returns only
  * one of them.  Different plugins should be given different basenames. */
-EXPORT PluginHandle * aud_plugin_lookup_basename (const char * basename)
+static PluginHandle * plugin_lookup_basename (const char * basename, bool compat_only)
 {
-    for (auto & list : plugins)
+    for (auto & list : (compat_only ? compatible : plugins))
     {
         for (PluginHandle * plugin : list)
         {
@@ -454,10 +489,16 @@ EXPORT PluginHandle * aud_plugin_lookup_basename (const char * basename)
     return nullptr;
 }
 
+EXPORT PluginHandle * aud_plugin_lookup_basename (const char * basename)
+{
+    return plugin_lookup_basename (basename, true);
+}
+
 static void plugin_get_info (PluginHandle * plugin, bool is_new)
 {
     Plugin * header = plugin->header;
 
+    plugin->version = header->version;
     plugin->name = String (header->info.name);
     plugin->domain = String (header->info.domain);
     plugin->has_about = (bool) header->info.about;
@@ -478,6 +519,8 @@ static void plugin_get_info (PluginHandle * plugin, bool is_new)
         plugin->exts.clear ();
         for (const char * ext : pp->extensions)
             plugin->exts.append (String (ext));
+
+        plugin->can_save = pp->can_save;
     }
     else if (header->type == PluginType::Input)
     {
@@ -507,8 +550,8 @@ static void plugin_get_info (PluginHandle * plugin, bool is_new)
     else if (header->type == PluginType::General)
     {
         GeneralPlugin * gp = (GeneralPlugin *) header;
-        if (is_new)
-            plugin->enabled = gp->enabled_by_default;
+        if (is_new && gp->enabled_by_default)
+            plugin->enabled = PluginEnabled::Primary;
     }
 }
 
@@ -518,7 +561,7 @@ void plugin_register (const char * path, int timestamp)
     if (! basename)
         return;
 
-    PluginHandle * plugin = aud_plugin_lookup_basename (basename);
+    PluginHandle * plugin = plugin_lookup_basename (basename, false);
 
     if (plugin)
     {
@@ -546,7 +589,8 @@ void plugin_register (const char * path, int timestamp)
         if (! header)
             return;
 
-        plugin = new PluginHandle (basename, path, true, timestamp, header->type, header);
+        plugin = new PluginHandle (basename, path, true, timestamp,
+         header->version, header->type, header);
         plugins[plugin->type].append (plugin);
 
         plugin_get_info (plugin, true);
@@ -570,21 +614,19 @@ EXPORT const void * aud_plugin_get_header (PluginHandle * plugin)
     if (! plugin->loaded)
     {
         Plugin * header = plugin_load (plugin->path);
-        if (! header || header->type != plugin->type)
-            goto DONE;
+        if (header && header->type == plugin->type)
+            plugin->header = header;
 
         plugin->loaded = true;
-        plugin->header = header;
     }
 
-DONE:
     pthread_mutex_unlock (& mutex);
     return plugin->header;
 }
 
 EXPORT PluginHandle * aud_plugin_by_header (const void * header)
 {
-    for (auto & list : plugins)
+    for (auto & list : compatible)
     {
         for (PluginHandle * plugin : list)
         {
@@ -598,7 +640,7 @@ EXPORT PluginHandle * aud_plugin_by_header (const void * header)
 
 EXPORT const Index<PluginHandle *> & aud_plugin_list (PluginType type)
 {
-    return plugins[type];
+    return compatible[type];
 }
 
 EXPORT const char * aud_plugin_get_name (PluginHandle * plugin)
@@ -618,7 +660,7 @@ EXPORT bool aud_plugin_has_configure (PluginHandle * plugin)
 
 EXPORT bool aud_plugin_get_enabled (PluginHandle * plugin)
 {
-    return plugin->enabled;
+    return plugin->enabled != PluginEnabled::Disabled;
 }
 
 static void plugin_call_watches (PluginHandle * plugin)
@@ -629,7 +671,12 @@ static void plugin_call_watches (PluginHandle * plugin)
     plugin->watches.remove_if (call_and_check_remove);
 }
 
-void plugin_set_enabled (PluginHandle * plugin, bool enabled)
+PluginEnabled plugin_get_enabled (PluginHandle * plugin)
+{
+    return plugin->enabled;
+}
+
+void plugin_set_enabled (PluginHandle * plugin, PluginEnabled enabled)
 {
     plugin->enabled = enabled;
     plugin_call_watches (plugin);
@@ -638,6 +685,7 @@ void plugin_set_enabled (PluginHandle * plugin, bool enabled)
 void plugin_set_failed (PluginHandle * plugin)
 {
     plugin->header = nullptr;
+    plugin_set_enabled (plugin, PluginEnabled::Disabled);
 }
 
 EXPORT void aud_plugin_add_watch (PluginHandle * plugin, PluginWatchFunc func, void * data)
@@ -655,8 +703,6 @@ EXPORT void aud_plugin_remove_watch (PluginHandle * plugin, PluginWatchFunc func
 
 bool transport_plugin_has_scheme (PluginHandle * plugin, const char * scheme)
 {
-    g_return_val_if_fail (plugin->type == PluginType::Transport, false);
-
     for (String & s : plugin->schemes)
     {
         if (! strcmp (s, scheme))
@@ -666,10 +712,18 @@ bool transport_plugin_has_scheme (PluginHandle * plugin, const char * scheme)
     return false;
 }
 
+bool playlist_plugin_can_save (PluginHandle * plugin)
+{
+    return plugin->can_save;
+}
+
+const Index<String> & playlist_plugin_get_exts (PluginHandle * plugin)
+{
+    return plugin->exts;
+}
+
 bool playlist_plugin_has_ext (PluginHandle * plugin, const char * ext)
 {
-    g_return_val_if_fail (plugin->type == PluginType::Playlist, false);
-
     for (String & e : plugin->exts)
     {
         if (! strcmp_nocase (e, ext))
@@ -681,8 +735,6 @@ bool playlist_plugin_has_ext (PluginHandle * plugin, const char * ext)
 
 bool input_plugin_has_key (PluginHandle * plugin, InputKey key, const char * value)
 {
-    g_return_val_if_fail (plugin->type == PluginType::Input, false);
-
     for (String & s : plugin->keys[key])
     {
         if (! strcmp_nocase (s, value))
@@ -694,12 +746,10 @@ bool input_plugin_has_key (PluginHandle * plugin, InputKey key, const char * val
 
 bool input_plugin_has_subtunes (PluginHandle * plugin)
 {
-    g_return_val_if_fail (plugin->type == PluginType::Input, false);
     return plugin->has_subtunes;
 }
 
 bool input_plugin_can_write_tuple (PluginHandle * plugin)
 {
-    g_return_val_if_fail (plugin->type == PluginType::Input, false);
     return plugin->writes_tag;
 }
