@@ -33,14 +33,12 @@
 #include <libaudcore/preferences.h>
 #include <libaudcore/runtime.h>
 
+#include "libguess/libguess.h"
+
 #include "internal.h"
 #include "libaudgui.h"
 #include "libaudgui-gtk.h"
 
-#ifdef USE_CHARDET
-#include <libguess.h>
-#endif
-
 enum CategoryViewCols {
     CATEGORY_VIEW_COL_ICON,
     CATEGORY_VIEW_COL_NAME,
@@ -111,7 +109,6 @@ static const TitleFieldTag title_field_tags[] = {
     { N_("Quality")    , "${quality}" }
 };
 
-#ifdef USE_CHARDET
 static const ComboItem chardet_detector_presets[] = {
     ComboItem (N_("None"), ""),
     ComboItem (N_("Arabic"), GUESS_REGION_AR),
@@ -126,7 +123,6 @@ static const ComboItem chardet_detector_presets[] = {
     ComboItem (N_("Taiwanese"), GUESS_REGION_TW),
     ComboItem (N_("Turkish"), GUESS_REGION_TR)
 };
-#endif
 
 static const ComboItem bitdepth_elements[] = {
     ComboItem ("16", 16),
@@ -170,6 +166,30 @@ static const PreferencesWidget output_combo_widgets[] = {
     WidgetCustomGTK (output_create_about_button)
 };
 
+static GtkWidget * record_checkbox;
+static GtkWidget * record_config_button;
+static GtkWidget * record_about_button;
+
+static void * record_create_checkbox ();
+static void * record_create_config_button ();
+static void * record_create_about_button ();
+
+static const PreferencesWidget record_buttons[] = {
+    WidgetCustomGTK (record_create_config_button),
+    WidgetCustomGTK (record_create_about_button)
+};
+
+static const PreferencesWidget gain_table[] = {
+    WidgetSpin (N_("Amplify all files:"),
+        WidgetFloat (0, "replay_gain_preamp"),
+        {-15, 15, 0.1, N_("dB")},
+        WIDGET_CHILD),
+    WidgetSpin (N_("Amplify untagged files:"),
+        WidgetFloat (0, "default_gain"),
+        {-15, 15, 0.1, N_("dB")},
+        WIDGET_CHILD)
+};
+
 static const PreferencesWidget audio_page_widgets[] = {
     WidgetLabel (N_("<b>Output Settings</b>")),
     WidgetBox ({{output_combo_widgets}, true}),
@@ -179,6 +199,9 @@ static const PreferencesWidget audio_page_widgets[] = {
     WidgetSpin (N_("Buffer size:"),
         WidgetInt (0, "output_buffer_size"),
         {100, 10000, 1000, N_("ms")}),
+    WidgetCustomGTK (record_create_checkbox),
+    WidgetBox ({{record_buttons}, true},
+        WIDGET_CHILD),
     WidgetCheck (N_("Soft clipping"),
         WidgetBool (0, "soft_clipping")),
     WidgetCheck (N_("Use software volume control (not recommended)"),
@@ -194,13 +217,7 @@ static const PreferencesWidget audio_page_widgets[] = {
         WIDGET_CHILD),
     WidgetLabel (N_("<b>Adjust Levels</b>"),
         WIDGET_CHILD),
-    WidgetSpin (N_("Amplify all files:"),
-        WidgetFloat (0, "replay_gain_preamp"),
-        {-15, 15, 0.1, N_("dB")},
-        WIDGET_CHILD),
-    WidgetSpin (N_("Amplify untagged files:"),
-        WidgetFloat (0, "default_gain"),
-        {-15, 15, 0.1, N_("dB")},
+    WidgetTable ({{gain_table}},
         WIDGET_CHILD)
 };
 
@@ -236,11 +253,9 @@ static const PreferencesWidget connectivity_page_widgets[] = {
 };
 
 static const PreferencesWidget chardet_elements[] = {
-#ifdef USE_CHARDET
     WidgetCombo (N_("Auto character encoding detector for:"),
         WidgetString (0, "chardet_detector"),
         {{chardet_detector_presets}}),
-#endif
     WidgetEntry (N_("Fallback character encodings:"),
         WidgetString (0, "chardet_fallback"))
 };
@@ -261,21 +276,16 @@ static const PreferencesWidget playlist_page_widgets[] = {
         WidgetBool (0, "clear_playlist")),
     WidgetCheck (N_("Open files in a temporary playlist"),
         WidgetBool (0, "open_to_temporary")),
-    WidgetLabel (N_("<b>Compatibility</b>")),
-    WidgetCheck (N_("Interpret \\ (backward slash) as a folder delimiter"),
-        WidgetBool (0, "convert_backslash")),
-    WidgetTable ({{chardet_elements}}),
     WidgetLabel (N_("<b>Song Display</b>")),
     WidgetCheck (N_("Show song numbers"),
         WidgetBool (0, "show_numbers_in_pl", send_title_change)),
     WidgetCheck (N_("Show leading zeroes (02:00 instead of 2:00)"),
         WidgetBool (0, "leading_zero", send_title_change)),
     WidgetCustomGTK (create_titlestring_table),
-    WidgetLabel (N_("<b>Advanced</b>")),
-    WidgetCheck (N_("Do not load metadata for songs until played"),
-        WidgetBool (0, "metadata_on_play")),
-    WidgetCheck (N_("Probe content of files with no recognized file name extension"),
-        WidgetBool (0, "slow_probe"))
+    WidgetLabel (N_("<b>Compatibility</b>")),
+    WidgetCheck (N_("Interpret \\ (backward slash) as a folder delimiter"),
+        WidgetBool (0, "convert_backslash")),
+    WidgetTable ({{chardet_elements}}),
 };
 
 static const PreferencesWidget song_info_page_widgets[] = {
@@ -301,13 +311,22 @@ static const PreferencesWidget song_info_page_widgets[] = {
         WIDGET_CHILD),
     WidgetCheck (N_("Show time scale for current song"),
         WidgetBool (0, "filepopup_showprogressbar"),
-        WIDGET_CHILD)
+        WIDGET_CHILD),
+    WidgetLabel (N_("<b>Advanced</b>")),
+    WidgetCheck (N_("Guess missing metadata from file path"),
+        WidgetBool (0, "metadata_fallbacks")),
+    WidgetCheck (N_("Do not load metadata for songs until played"),
+        WidgetBool (0, "metadata_on_play")),
+    WidgetCheck (N_("Probe content of files with no recognized file name extension"),
+        WidgetBool (0, "slow_probe"))
 };
 
-#define TITLESTRING_NPRESETS 6
+#define TITLESTRING_NPRESETS 8
 
 static const char * const titlestring_presets[TITLESTRING_NPRESETS] = {
     "${title}",
+    "${title}${?artist: - ${artist}}",
+    "${title}${?artist: - ${artist}}${?album: - ${album}}",
     "${?artist:${artist} - }${title}",
     "${?artist:${artist} - }${?album:${album} - }${title}",
     "${?artist:${artist} - }${?album:${album} - }${?track-number:${track-number}. }${title}",
@@ -317,6 +336,8 @@ static const char * const titlestring_presets[TITLESTRING_NPRESETS] = {
 
 static const char * const titlestring_preset_names[TITLESTRING_NPRESETS] = {
     N_("TITLE"),
+    N_("TITLE - ARTIST"),
+    N_("TITLE - ARTIST - ALBUM"),
     N_("ARTIST - TITLE"),
     N_("ARTIST - ALBUM - TITLE"),
     N_("ARTIST - ALBUM - TRACK. TITLE"),
@@ -427,9 +448,6 @@ static void fill_category_list (GtkTreeView * treeview, GtkNotebook * notebook)
     gtk_tree_view_column_pack_start (column, renderer, false);
     gtk_tree_view_column_set_attributes (column, renderer, "text", 1, nullptr);
 
-    g_object_set ((GObject *) renderer, "wrap-width", 106, "wrap-mode",
-     PANGO_WRAP_WORD_CHAR, nullptr);
-
     GtkListStore * store = gtk_list_store_new (CATEGORY_VIEW_N_COLS,
      GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_INT);
     gtk_tree_view_set_model (treeview, (GtkTreeModel *) store);
@@ -527,7 +545,7 @@ static void * create_titlestring_table ()
     g_signal_connect (titlestring_help_button, "clicked",
      (GCallback) on_titlestring_help_button_clicked, titlestring_tag_menu);
 
-    GtkWidget * image = gtk_image_new_from_icon_name ("list-add", GTK_ICON_SIZE_BUTTON);
+    GtkWidget * image = gtk_image_new_from_icon_name ("list-add", GTK_ICON_SIZE_MENU);
     gtk_container_add ((GtkContainer *) titlestring_help_button, image);
 
     return grid;
@@ -555,16 +573,6 @@ static void iface_fill_prefs_box ()
          header->info.prefs->widgets, header->info.domain);
 }
 
-static int iface_combo_changed_finish (void *)
-{
-    iface_fill_prefs_box ();
-    gtk_widget_show_all (iface_prefs_box);
-
-    audgui_cleanup ();
-
-    return G_SOURCE_REMOVE;
-}
-
 static void iface_combo_changed ()
 {
     /* prevent audgui from being shut down during the switch */
@@ -575,8 +583,10 @@ static void iface_combo_changed ()
 
     aud_plugin_enable (aud_plugin_list (PluginType::Iface)[iface_combo_selected], true);
 
-    /* now wait till we have restarted into the new main loop */
-    g_idle_add_full (G_PRIORITY_HIGH, iface_combo_changed_finish, nullptr, nullptr);
+    iface_fill_prefs_box ();
+    gtk_widget_show_all (iface_prefs_box);
+
+    audgui_cleanup ();
 }
 
 static ArrayRef<ComboItem> iface_combo_fill ()
@@ -633,22 +643,15 @@ static void output_bit_depth_changed ()
     aud_output_reset (OutputReset::ReopenStream);
 }
 
-static void output_do_config (void *)
-{
-    audgui_show_plugin_prefs (aud_plugin_get_current (PluginType::Output));
-}
-
-static void output_do_about (void *)
-{
-    audgui_show_plugin_about (aud_plugin_get_current (PluginType::Output));
-}
-
 static void * output_create_config_button ()
 {
-    gboolean enabled = aud_plugin_has_configure (aud_plugin_get_current (PluginType::Output));
+    auto do_config = [] (void *)
+        { audgui_show_plugin_prefs (aud_plugin_get_current (PluginType::Output)); };
+
+    bool enabled = aud_plugin_has_configure (aud_plugin_get_current (PluginType::Output));
 
     output_config_button = audgui_button_new (_("_Settings"),
-     "preferences-system", output_do_config, nullptr);
+     "preferences-system", do_config, nullptr);
     gtk_widget_set_sensitive (output_config_button, enabled);
 
     return output_config_button;
@@ -656,14 +659,79 @@ static void * output_create_config_button ()
 
 static void * output_create_about_button ()
 {
-    gboolean enabled = aud_plugin_has_about (aud_plugin_get_current (PluginType::Output));
+    auto do_about = [] (void *)
+        { audgui_show_plugin_about (aud_plugin_get_current (PluginType::Output)); };
 
-    output_about_button = audgui_button_new (_("_About"), "help-about", output_do_about, nullptr);
+    bool enabled = aud_plugin_has_about (aud_plugin_get_current (PluginType::Output));
+
+    output_about_button = audgui_button_new (_("_About"), "help-about", do_about, nullptr);
     gtk_widget_set_sensitive (output_about_button, enabled);
 
     return output_about_button;
 }
 
+static void record_toggled (GtkToggleButton * button)
+{
+    aud_drct_enable_record (gtk_toggle_button_get_active (button));
+}
+
+static void * record_create_checkbox ()
+{
+    record_checkbox = gtk_check_button_new ();
+    g_signal_connect (record_checkbox, "toggled", (GCallback) record_toggled, nullptr);
+    return record_checkbox;
+}
+
+static void * record_create_config_button ()
+{
+    auto do_config = [] (void *)
+    {
+        if (aud_drct_get_record_enabled ())
+            audgui_show_plugin_prefs (aud_drct_get_record_plugin ());
+    };
+
+    return (record_config_button = audgui_button_new (_("_Settings"),
+     "preferences-system", do_config, nullptr));
+}
+
+static void * record_create_about_button ()
+{
+    auto do_about = [] (void *)
+    {
+        if (aud_drct_get_record_enabled ())
+            audgui_show_plugin_about (aud_drct_get_record_plugin ());
+    };
+
+    return (record_about_button = audgui_button_new (_("_About"), "help-about",
+     do_about, nullptr));
+}
+
+static void record_update (void * = nullptr, void * = nullptr)
+{
+    auto p = aud_drct_get_record_plugin ();
+
+    if (p)
+    {
+        bool enabled = aud_drct_get_record_enabled ();
+
+        gtk_widget_set_sensitive (record_checkbox, true);
+        gtk_button_set_label ((GtkButton *) record_checkbox,
+         str_printf (_("Record audio stream using %s"), aud_plugin_get_name (p)));
+        gtk_toggle_button_set_active ((GtkToggleButton *) record_checkbox, enabled);
+        gtk_widget_set_sensitive (record_config_button, enabled && aud_plugin_has_configure (p));
+        gtk_widget_set_sensitive (record_about_button, enabled && aud_plugin_has_about (p));
+    }
+    else
+    {
+        gtk_widget_set_sensitive (record_checkbox, false);
+        gtk_button_set_label ((GtkButton *) record_checkbox,
+         str_printf (_("No audio recording plugin available")));
+        gtk_toggle_button_set_active ((GtkToggleButton *) record_checkbox, false);
+        gtk_widget_set_sensitive (record_config_button, false);
+        gtk_widget_set_sensitive (record_about_button, false);
+    }
+}
+
 static void create_audio_category ()
 {
     GtkWidget * audio_page_vbox = gtk_vbox_new (false, 0);
@@ -694,6 +762,8 @@ static void create_plugin_category ()
 
 static void destroy_cb ()
 {
+    hook_dissociate ("enable record", record_update);
+
     prefswin = nullptr;
     category_treeview = nullptr;
     category_notebook = nullptr;
@@ -709,7 +779,6 @@ static void create_prefs_window ()
     gtk_window_set_type_hint ((GtkWindow *) prefswin, GDK_WINDOW_TYPE_HINT_DIALOG);
     gtk_container_set_border_width ((GtkContainer *) prefswin, 12);
     gtk_window_set_title ((GtkWindow *) prefswin, _("Audacious Settings"));
-    gtk_window_set_default_size ((GtkWindow *) prefswin, 680, 400);
 
     GtkWidget * vbox = gtk_vbox_new (false, 0);
     gtk_container_add ((GtkContainer *) prefswin, vbox);
@@ -725,7 +794,7 @@ static void create_prefs_window ()
 
     category_treeview = gtk_tree_view_new ();
     gtk_container_add ((GtkContainer *) scrolledwindow, category_treeview);
-    gtk_widget_set_size_request (scrolledwindow, 168, -1);
+    gtk_widget_set_size_request (scrolledwindow, audgui_get_dpi () * 7 / 4, -1);
     gtk_tree_view_set_headers_visible ((GtkTreeView *) category_treeview, false);
 
     category_notebook = gtk_notebook_new ();
@@ -766,6 +835,9 @@ static void create_prefs_window ()
 
     fill_category_list ((GtkTreeView *) category_treeview, (GtkNotebook *) category_notebook);
 
+    record_update ();
+    hook_associate ("enable record", record_update, nullptr);
+
     gtk_widget_show_all (vbox);
 
     g_signal_connect (prefswin, "destroy", (GCallback) destroy_cb, nullptr);
