@@ -33,7 +33,8 @@ struct TupleFieldMap {
     Tuple::Field field;
     bool editable;
 };
-const TupleFieldMap tuple_field_map[] = {
+
+static const TupleFieldMap tuple_field_map[] = {
     {N_("Metadata"), Tuple::Invalid, false},
     {N_("Artist"), Tuple::Artist, true},
     {N_("Album"), Tuple::Album, true},
@@ -47,7 +48,7 @@ const TupleFieldMap tuple_field_map[] = {
     {N_("Recording Year"), Tuple::Year, true},
     {N_("Recording Date"), Tuple::Date, true},
 
-    {"", Tuple::Invalid, false},
+    {nullptr, Tuple::Invalid, false},
     {N_("Technical"), Tuple::Invalid, false},
     {N_("Length"), Tuple::Length, false},
     {N_("MIME Type"), Tuple::MIMEType, false},
@@ -60,8 +61,8 @@ EXPORT InfoWidget::InfoWidget (QWidget * parent) : QTreeView (parent)
 {
     setModel (& m_model);
     header ()->hide ();
-    setEditTriggers (QAbstractItemView::SelectedClicked);
     setIndentation (0);
+    resizeColumnToContents (0);
 }
 
 EXPORT InfoWidget::~InfoWidget ()
@@ -72,6 +73,8 @@ EXPORT void InfoWidget::fillInfo (int playlist, int entry, const char * filename
  PluginHandle * decoder, bool updating_enabled)
 {
     m_model.setTupleData (tuple, String (filename), decoder);
+    reset ();
+    setEditTriggers (updating_enabled ? QAbstractItemView::SelectedClicked : QAbstractItemView::NoEditTriggers);
 }
 
 EXPORT bool InfoWidget::updateFile ()
