@@ -41,19 +41,18 @@
 #include <libaudcore/drct.h>
 #include <libaudcore/hook.h>
 #include <libaudcore/i18n.h>
+#include <libaudcore/mainloop.h>
 #include <libaudcore/playlist.h>
 #include <libaudcore/plugin.h>
 #include <libaudcore/plugins.h>
 #include <libaudcore/preferences.h>
 #include <libaudcore/runtime.h>
 
+#include "libguess/libguess.h"
+
 #include "libaudqt.h"
 #include "prefs-pluginlist-model.h"
 
-#ifdef USE_CHARDET
-#include <libguess.h>
-#endif
-
 namespace audqt {
 
 struct Category {
@@ -105,8 +104,8 @@ static const TitleFieldTag title_field_tags[] = {
     { N_("Title")      , "${title}" },
     { N_("Track number"), "${track-number}" },
     { N_("Genre")      , "${genre}" },
-    { N_("File name")   , "${file-name}" },
-    { N_("File path")   , "${file-path}" },
+    { N_("File name")  , "${file-name}" },
+    { N_("File path")  , "${file-path}" },
     { N_("Date")       , "${date}" },
     { N_("Year")       , "${year}" },
     { N_("Comment")    , "${comment}" },
@@ -114,7 +113,6 @@ static const TitleFieldTag title_field_tags[] = {
     { N_("Quality")    , "${quality}" }
 };
 
-#ifdef USE_CHARDET
 static const ComboItem chardet_detector_presets[] = {
     ComboItem (N_("None"), ""),
     ComboItem (N_("Arabic"), GUESS_REGION_AR),
@@ -129,7 +127,6 @@ static const ComboItem chardet_detector_presets[] = {
     ComboItem (N_("Taiwanese"), GUESS_REGION_TW),
     ComboItem (N_("Turkish"), GUESS_REGION_TR)
 };
-#endif
 
 static const ComboItem bitdepth_elements[] = {
     ComboItem ("16", 16),
@@ -143,8 +140,8 @@ static int iface_combo_selected;
 static QWidget * iface_prefs_box;
 
 static ArrayRef<ComboItem> iface_combo_fill ();
-static void iface_combo_changed (void);
-static void * iface_create_prefs_box (void);
+static void iface_combo_changed ();
+static void * iface_create_prefs_box ();
 
 static const PreferencesWidget appearance_page_widgets[] = {
     WidgetLabel (N_("<b>Interface Settings</b>")),
@@ -160,10 +157,10 @@ static QPushButton * output_config_button;
 static QPushButton * output_about_button;
 
 static ArrayRef<ComboItem> output_combo_fill ();
-static void output_combo_changed (void);
-static void * output_create_config_button (void);
-static void * output_create_about_button (void);
-static void output_bit_depth_changed (void);
+static void output_combo_changed ();
+static void * output_create_config_button ();
+static void * output_create_about_button ();
+static void output_bit_depth_changed ();
 
 static const PreferencesWidget output_combo_widgets[] = {
     WidgetCombo (N_("Output plugin:"),
@@ -173,6 +170,17 @@ static const PreferencesWidget output_combo_widgets[] = {
     WidgetCustomQt (output_create_about_button)
 };
 
+static const PreferencesWidget gain_table[] = {
+    WidgetSpin (N_("Amplify all files:"),
+        WidgetFloat (0, "replay_gain_preamp"),
+        {-15, 15, 0.1, N_("dB")},
+        WIDGET_CHILD),
+    WidgetSpin (N_("Amplify untagged files:"),
+        WidgetFloat (0, "default_gain"),
+        {-15, 15, 0.1, N_("dB")},
+        WIDGET_CHILD)
+};
+
 static const PreferencesWidget audio_page_widgets[] = {
     WidgetLabel (N_("<b>Output Settings</b>")),
     WidgetBox ({{output_combo_widgets}, true}),
@@ -197,13 +205,7 @@ static const PreferencesWidget audio_page_widgets[] = {
         WIDGET_CHILD),
     WidgetLabel (N_("<b>Adjust Levels</b>"),
         WIDGET_CHILD),
-    WidgetSpin (N_("Amplify all files:"),
-        WidgetFloat (0, "replay_gain_preamp"),
-        {-15, 15, 0.1, N_("dB")},
-        WIDGET_CHILD),
-    WidgetSpin (N_("Amplify untagged files:"),
-        WidgetFloat (0, "default_gain"),
-        {-15, 15, 0.1, N_("dB")},
+    WidgetTable ({{gain_table}},
         WIDGET_CHILD)
 };
 
@@ -239,18 +241,16 @@ static const PreferencesWidget connectivity_page_widgets[] = {
 };
 
 static const PreferencesWidget chardet_elements[] = {
-#ifdef USE_CHARDET
     WidgetCombo (N_("Auto character encoding detector for:"),
         WidgetString (0, "chardet_detector"),
         {{chardet_detector_presets}}),
-#endif
     WidgetEntry (N_("Fallback character encodings:"),
         WidgetString (0, "chardet_fallback"))
 };
 
 
-static void send_title_change (void);
-static void * create_titlestring_table (void);
+static void send_title_change ();
+static void * create_titlestring_table ();
 
 static const PreferencesWidget playlist_page_widgets[] = {
     WidgetLabel (N_("<b>Behavior</b>")),
@@ -267,21 +267,16 @@ static const PreferencesWidget playlist_page_widgets[] = {
         WidgetBool (0, "open_to_temporary")),
     WidgetCheck (N_("Do not load metadata for songs until played"),
         WidgetBool (0, "metadata_on_play")),
-    WidgetLabel (N_("<b>Compatibility</b>")),
-    WidgetCheck (N_("Interpret \\ (backward slash) as a folder delimiter"),
-        WidgetBool (0, "convert_backslash")),
-    WidgetTable ({{chardet_elements}}),
     WidgetLabel (N_("<b>Song Display</b>")),
     WidgetCheck (N_("Show song numbers"),
         WidgetBool (0, "show_numbers_in_pl", send_title_change)),
     WidgetCheck (N_("Show leading zeroes (02:00 instead of 2:00)"),
         WidgetBool (0, "leading_zero", send_title_change)),
     WidgetCustomQt (create_titlestring_table),
-    WidgetLabel (N_("<b>Advanced</b>")),
-    WidgetCheck (N_("Do not load metadata for songs until played"),
-        WidgetBool (0, "metadata_on_play")),
-    WidgetCheck (N_("Probe content of files with no recognized file name extension"),
-        WidgetBool (0, "slow_probe"))
+    WidgetLabel (N_("<b>Compatibility</b>")),
+    WidgetCheck (N_("Interpret \\ (backward slash) as a folder delimiter"),
+        WidgetBool (0, "convert_backslash")),
+    WidgetTable ({{chardet_elements}})
 };
 
 static const PreferencesWidget song_info_page_widgets[] = {
@@ -307,13 +302,22 @@ static const PreferencesWidget song_info_page_widgets[] = {
         WIDGET_CHILD),
     WidgetCheck (N_("Show time scale for current song"),
         WidgetBool (0, "filepopup_showprogressbar"),
-        WIDGET_CHILD)
+        WIDGET_CHILD),
+    WidgetLabel (N_("<b>Advanced</b>")),
+    WidgetCheck (N_("Guess missing metadata from file path"),
+        WidgetBool (0, "metadata_fallbacks")),
+    WidgetCheck (N_("Do not load metadata for songs until played"),
+        WidgetBool (0, "metadata_on_play")),
+    WidgetCheck (N_("Probe content of files with no recognized file name extension"),
+        WidgetBool (0, "slow_probe"))
 };
 
-#define TITLESTRING_NPRESETS 6
+#define TITLESTRING_NPRESETS 8
 
 static const char * const titlestring_presets[TITLESTRING_NPRESETS] = {
     "${title}",
+    "${title}${?artist: - ${artist}}",
+    "${title}${?artist: - ${artist}}${?album: - ${album}}",
     "${?artist:${artist} - }${title}",
     "${?artist:${artist} - }${?album:${album} - }${title}",
     "${?artist:${artist} - }${?album:${album} - }${?track-number:${track-number}. }${title}",
@@ -323,6 +327,8 @@ static const char * const titlestring_presets[TITLESTRING_NPRESETS] = {
 
 static const char * const titlestring_preset_names[TITLESTRING_NPRESETS] = {
     N_("TITLE"),
+    N_("TITLE - ARTIST"),
+    N_("TITLE - ARTIST - ALBUM"),
     N_("ARTIST - TITLE"),
     N_("ARTIST - ALBUM - TITLE"),
     N_("ARTIST - ALBUM - TRACK. TITLE"),
@@ -330,7 +336,7 @@ static const char * const titlestring_preset_names[TITLESTRING_NPRESETS] = {
     N_("ALBUM - TITLE")
 };
 
-static void * create_titlestring_table (void)
+static void * create_titlestring_table ()
 {
     QWidget * w = new QWidget;
     QGridLayout * l = new QGridLayout (w);
@@ -341,7 +347,7 @@ static void * create_titlestring_table (void)
     QComboBox * cbox = new QComboBox (w);
     l->addWidget (cbox, 0, 1);
 
-    for (int i = 0; i < TITLESTRING_NPRESETS; i++)
+    for (int i = 0; i < TITLESTRING_NPRESETS; i ++)
         cbox->addItem (translate_str (titlestring_preset_names [i]), i);
     cbox->addItem (_("Custom"), TITLESTRING_NPRESETS);
     cbox->setCurrentIndex (TITLESTRING_NPRESETS);
@@ -354,12 +360,10 @@ static void * create_titlestring_table (void)
 
     String format = aud_get_str (nullptr, "generic_title_format");
     le->setText ((const char *) format);
-    for (int i = 0; i < TITLESTRING_NPRESETS; i++)
+    for (int i = 0; i < TITLESTRING_NPRESETS; i ++)
     {
         if (! strcmp (titlestring_presets [i], format))
-        {
             cbox->setCurrentIndex (i);
-        }
     }
 
     QObject::connect (le, &QLineEdit::textChanged, [=] (const QString & text) {
@@ -375,7 +379,9 @@ static void * create_titlestring_table (void)
 
     /* build menu */
     QPushButton * btn_mnu = new QPushButton (w);
+    btn_mnu->setFixedWidth (btn_mnu->sizeHint ().height ());
     btn_mnu->setIcon (QIcon::fromTheme ("list-add"));
+    btn_mnu->setIconSize (QSize (16, 16));
     l->addWidget (btn_mnu, 1, 2);
 
     QMenu * mnu_fields = new QMenu (w);
@@ -383,12 +389,12 @@ static void * create_titlestring_table (void)
     for (auto & t : title_field_tags)
     {
         QAction * a = mnu_fields->addAction (_(t.name));
-        QObject::connect (a, &QAction::triggered, [=] () {
+        QObject::connect (a, & QAction::triggered, [=] () {
             le->insert (t.tag);
         });
     }
 
-    QObject::connect (btn_mnu, &QAbstractButton::clicked, [=] () {
+    QObject::connect (btn_mnu, & QAbstractButton::clicked, [=] () {
         mnu_fields->popup (btn_mnu->mapToGlobal (QPoint (0, 0)));
     });
 
@@ -406,13 +412,13 @@ static Index<ComboItem> fill_plugin_combo (PluginType type)
     return elems;
 }
 
-static void send_title_change (void)
+static void send_title_change ()
 {
     if (aud_drct_get_ready ())
         hook_call ("title change", nullptr);
 }
 
-static void iface_fill_prefs_box (void)
+static void iface_fill_prefs_box ()
 {
     Plugin * header = (Plugin *) aud_plugin_get_header (aud_plugin_get_current (PluginType::Iface));
     if (header && header->info.prefs)
@@ -425,32 +431,21 @@ static void iface_fill_prefs_box (void)
     }
 }
 
-#ifdef XXX_NOTYET
-static int iface_combo_changed_finish (void *)
+static void iface_combo_changed ()
 {
+    /* prevent audqt from being shut down during the switch */
+    init ();
+
+    if (QLayout * layout = iface_prefs_box->layout ())
+    {
+        clear_layout (layout);
+        delete layout;
+    }
+
+    aud_plugin_enable (aud_plugin_list (PluginType::Iface)[iface_combo_selected], true);
+
     iface_fill_prefs_box ();
-    gtk_widget_show_all (iface_prefs_box);
-
-    audgui_cleanup ();
-
-    return G_SOURCE_REMOVE;
-}
-#endif
-
-static void iface_combo_changed (void)
-{
-#ifdef XXX_NOTYET
-    /* prevent audgui from being shut down during the switch */
-    audgui_init ();
-
-    gtk_container_foreach ((GtkContainer *) iface_prefs_box,
-     (GtkCallback) gtk_widget_destroy, nullptr);
-
-    aud_plugin_enable (aud_plugin_by_index (PluginType::Iface, iface_combo_selected), true);
-
-    /* now wait till we have restarted into the new main loop */
-    g_idle_add_full (G_PRIORITY_HIGH, iface_combo_changed_finish, nullptr, nullptr);
-#endif
+    cleanup ();
 }
 
 static ArrayRef<ComboItem> iface_combo_fill ()
@@ -465,14 +460,14 @@ static ArrayRef<ComboItem> iface_combo_fill ()
     return {iface_combo_elements.begin (), iface_combo_elements.len ()};
 }
 
-static void * iface_create_prefs_box (void)
+static void * iface_create_prefs_box ()
 {
     iface_prefs_box = new QWidget;
     iface_fill_prefs_box ();
     return iface_prefs_box;
 }
 
-static void output_combo_changed (void)
+static void output_combo_changed ()
 {
     PluginHandle * plugin = aud_plugin_list (PluginType::Output)[output_combo_selected];
 
@@ -483,21 +478,21 @@ static void output_combo_changed (void)
     }
 }
 
-static void * output_create_config_button (void)
+static void * output_create_config_button ()
 {
     bool enabled = aud_plugin_has_configure (aud_plugin_get_current (PluginType::Output));
 
     output_config_button = new QPushButton (translate_str (N_("_Settings")));
     output_config_button->setEnabled (enabled);
 
-    QObject::connect (output_config_button, &QAbstractButton::clicked, [=] (bool) {
+    QObject::connect (output_config_button, & QAbstractButton::clicked, [=] (bool) {
         plugin_prefs (aud_plugin_get_current (PluginType::Output));
     });
 
     return output_config_button;
 }
 
-static void * output_create_about_button (void)
+static void * output_create_about_button ()
 {
     bool enabled = aud_plugin_has_about (aud_plugin_get_current (PluginType::Output));
 
@@ -523,70 +518,22 @@ static ArrayRef<ComboItem> output_combo_fill ()
     return {output_combo_elements.begin (), output_combo_elements.len ()};
 }
 
-static void output_bit_depth_changed (void)
+static void output_bit_depth_changed ()
 {
     aud_output_reset (OutputReset::ReopenStream);
 }
 
-static void create_appearance_category (QStackedWidget * category_notebook)
+static void create_category (QStackedWidget * notebook, ArrayRef<PreferencesWidget> widgets)
 {
     QWidget * w = new QWidget;
     QVBoxLayout * vbox = new QVBoxLayout (w);
 
     vbox->setContentsMargins (0, 0, 0, 0);
     vbox->setSpacing (4);
-    prefs_populate (vbox, appearance_page_widgets, nullptr);
+    prefs_populate (vbox, widgets, nullptr);
+    vbox->addStretch (1);
 
-    category_notebook->addWidget (w);
-}
-
-static void create_audio_category (QStackedWidget * category_notebook)
-{
-    QWidget * audio_page = new QWidget;
-    QVBoxLayout * audio_page_vbox = new QVBoxLayout;
-
-    audio_page_vbox->setContentsMargins (0, 0, 0, 0);
-    audio_page_vbox->setSpacing (4);
-    prefs_populate (audio_page_vbox, audio_page_widgets, nullptr);
-
-    audio_page->setLayout (audio_page_vbox);
-    category_notebook->addWidget (audio_page);
-}
-
-static void create_connectivity_category (QStackedWidget * category_notebook)
-{
-    QWidget * connectivity_page = new QWidget;
-    QVBoxLayout * connectivity_page_vbox = new QVBoxLayout (connectivity_page);
-
-    connectivity_page_vbox->setContentsMargins (0, 0, 0, 0);
-    connectivity_page_vbox->setSpacing (4);
-    prefs_populate (connectivity_page_vbox, connectivity_page_widgets, nullptr);
-
-    category_notebook->addWidget (connectivity_page);
-}
-
-static void create_playlist_category (QStackedWidget * category_notebook)
-{
-    QWidget * playlist_page = new QWidget;
-    QVBoxLayout * playlist_page_vbox = new QVBoxLayout (playlist_page);
-
-    playlist_page_vbox->setContentsMargins (0, 0, 0, 0);
-    playlist_page_vbox->setSpacing (4);
-    prefs_populate (playlist_page_vbox, playlist_page_widgets, nullptr);
-
-    category_notebook->addWidget (playlist_page);
-}
-
-static void create_song_info_category (QStackedWidget * category_notebook)
-{
-    QWidget * song_info_page = new QWidget;
-    QVBoxLayout * song_info_page_vbox = new QVBoxLayout (song_info_page);
-
-    song_info_page_vbox->setContentsMargins (0, 0, 0, 0);
-    song_info_page_vbox->setSpacing (4);
-    prefs_populate (song_info_page_vbox, song_info_page_widgets, nullptr);
-
-    category_notebook->addWidget (song_info_page);
+    notebook->addWidget (w);
 }
 
 static void create_plugin_category_page (PluginType category_id, const char * category_name, QTabWidget * parent)
@@ -595,7 +542,7 @@ static void create_plugin_category_page (PluginType category_id, const char * ca
     QHeaderView * header = view->header ();
 
     view->setIndentation (0);
-    view->setModel (new PluginListModel (nullptr, category_id));
+    view->setModel (new PluginListModel (view, category_id));
     view->setSelectionMode (view->NoSelection);
 
     header->hide ();
@@ -615,12 +562,12 @@ static void create_plugin_category_page (PluginType category_id, const char * ca
 
         switch (index.column ())
         {
-            case PluginListModel::AboutColumn:
-                plugin_about (list[row]);
-                break;
-            case PluginListModel::SettingsColumn:
-                plugin_prefs (list[row]);
-                break;
+        case PluginListModel::AboutColumn:
+            plugin_about (list[row]);
+            break;
+        case PluginListModel::SettingsColumn:
+            plugin_prefs (list[row]);
+            break;
         }
     });
 }
@@ -653,7 +600,6 @@ static void create_prefs_window ()
     });
 
     QVBoxLayout * vbox_parent = new QVBoxLayout (s_prefswin);
-
     vbox_parent->setSpacing (0);
     vbox_parent->setContentsMargins (0, 0, 0, 0);
 
@@ -668,35 +614,35 @@ static void create_prefs_window ()
     s_category_notebook = new QStackedWidget;
     child_vbox->addWidget (s_category_notebook);
 
-    create_appearance_category (s_category_notebook);
-    create_audio_category (s_category_notebook);
-    create_connectivity_category (s_category_notebook);
-    create_playlist_category (s_category_notebook);
-    create_song_info_category (s_category_notebook);
+    create_category (s_category_notebook, appearance_page_widgets);
+    create_category (s_category_notebook, audio_page_widgets);
+    create_category (s_category_notebook, connectivity_page_widgets);
+    create_category (s_category_notebook, playlist_page_widgets);
+    create_category (s_category_notebook, song_info_page_widgets);
     create_plugin_category (s_category_notebook);
 
     QDialogButtonBox * bbox = new QDialogButtonBox (QDialogButtonBox::Close);
     bbox->button (QDialogButtonBox::Close)->setText (translate_str (N_("_Close")));
     child_vbox->addWidget (bbox);
 
-    QObject::connect (bbox, &QDialogButtonBox::rejected, s_prefswin, &QObject::deleteLater);
+    QObject::connect (bbox, & QDialogButtonBox::rejected, s_prefswin, & QObject::deleteLater);
 
-    QSignalMapper * mapper = new QSignalMapper;
+    QSignalMapper * mapper = new QSignalMapper (s_prefswin);
     const char * data_dir = aud_get_path (AudPath::DataDir);
 
     QObject::connect (mapper, static_cast <void (QSignalMapper::*)(int)>(&QSignalMapper::mapped),
                       s_category_notebook, static_cast <void (QStackedWidget::*)(int)>(&QStackedWidget::setCurrentIndex));
 
-    for (int i = 0; i < CATEGORY_COUNT; i++)
+    for (int i = 0; i < CATEGORY_COUNT; i ++)
     {
         QIcon ico (QString (filename_build ({data_dir, "images", categories[i].icon_path})));
-        QAction * a = new QAction (ico, translate_str (categories[i].name), nullptr);
+        QAction * a = new QAction (ico, translate_str (categories[i].name), toolbar);
 
         toolbar->addAction (a);
 
         mapper->setMapping (a, i);
 
-        QObject::connect (a, &QAction::triggered, mapper, static_cast <void (QSignalMapper::*)()>(&QSignalMapper::map));
+        QObject::connect (a, & QAction::triggered, mapper, static_cast <void (QSignalMapper::*)()>(& QSignalMapper::map));
     }
 }
 
